<?php

namespace MpApi\Controller;

use Yansongda\Pay\Pay;

class PayController extends PublicController
{
    /**
     * @description: 初始化 pay类的参数
     */
    private function init()
    {
        // 初始化 pay类的参数
        $this->config = [
            'wechat' => [
                'default' => [
                    'mch_id' => C('weixin')['mch_id'], // 必填-商户号
                    'mch_secret_key' => C('weixin')['mch_secret_key'], // 必填-v3商户秘钥
                    'mch_secret_cert' => C('weixin')['mch_secret_cert'], // 必填-商户私钥路径
                    'mch_public_cert_path' => C('weixin')['mch_public_cert_path'], // 必填-商户公钥证书路径
                    'notify_url' => C('host') . 'flycube.php/Api/Pay/notifyCallback', // 必填-支付回调通知地址
                    'refund_notify_url' => C('host') . 'flycube.php/Api/Pay/refundNotifyCallback', // 必填-退款回调通知地址
                    'mini_app_id' => C('weixin')['appid'], // 选填-小程序 的 app_id
                    'wechat_public_cert_path' => [ // 必填-微信平台公钥证书路径
                        '5878026B07BD819CCC25AC95FA31AE45BB6BC0D9' => C('weixin')['wechat_public_cert_path'],
                    ],
                ],
            ],
            'logger' => [ // 可选
                'enable' => true,
                'file' => C('weixin')['payLogger_path'],
                'level' => 'info', // 建议生产环境等级调整为 info，开发环境为 debug
                'type' => 'single', // 可选, 可选 daily
                'max_file' => 30, // 当 type 为 daily 时有效，默认 30 天
            ],
            'http' => [ // 可选
                'timeout' => 5.0,
                'connect_timeout' => 5.0,
            ],
        ];
    }

    /**
     * @description: Yansongda pay类的参数
     */
    protected $config;

    /**
     * @description: 发起退款
     */
    public function refund()
    {
        // 总管理员 可接收任何shop_id 非总管理员 只可以调用自身shop_id 否则会中断
        $this->isPower();
        // 初始化 pay类的参数
        $this->init();

        // 获取订单信息
        $where['shop_id'] = $_REQUEST['shop_id'];
        $where['order_sn'] = $_REQUEST['order_sn'];
        $field = array('order_sn, total_price, pay_sn');
        $orderDb = D('order');
        if (!$order = $orderDb->where($where)->field($field)->find()) {
            echo json_encode(array('status' => 0, 'msg' => '订单不存在'));
            exit();
        }
        if ($order['total_price'] < $_REQUEST['refund_price']) {
            echo json_encode(array('status' => 0, 'msg' => '退款申请超额'));
            exit();
        }

        // 设置退款参数
        $refundParameter = [
            'out_trade_no' => $order['order_sn'], // 商户订单号
            'out_refund_no' => $order['order_sn'] . '_refund', // 商户退款单号
            'amount' => [
                'refund' => $_REQUEST['refund_price'] * 100, // 退款金额，单位：分
                'total' => $order['total_price'] * 100, // 订单金额，单位：分
                'currency' => 'CNY',
            ],
        ];

        try {
            // 调用退款接口
            $result = Pay::wechat($this->config)->refund($refundParameter);
            // 处理退款结果
            echo json_encode(array('status' => 1, 'msg' => $result));
        } catch (\Exception $e) {
            echo json_encode(array('status' => 0, 'msg' => $e->getMessage()));
        }
    }
    /**
     * @description: 退款成功后 腾讯服务器的回调
     */
    public function refundNotifyCallback()
    {
        $this->init(); // 初始化 pay类的参数

        try {
            // 实例化 Yansongda Pay 并处理回调
            $result = Pay::wechat($this->config)->callback();

            // 验证成功，处理业务逻辑
            if ($result['resource']['ciphertext']['refund_status'] == 'SUCCESS') {
                $where['order_sn'] = $result['resource']['ciphertext']['out_trade_no']; // 获取订单号
                $orderDb = D('order'); // 实例化订单模型
                $data['main_status'] = '已退款'; // 更新订单状态为已退款
                $data['refund_time'] = time(); // 标记退款时间
                $data['refund_sn'] = $result['resource']['ciphertext']['refund_id']; // 退款单号
                // 更新订单
                $orderDb->where($where)->data($data)->save();
            }
        } catch (\Exception $e) {
            // 捕获并记录可能的异常
            error_log('退款回调处理错误：' . $e->getMessage());
        }

        // 返回成功响应给腾讯服务器，告知通知已处理
        return Pay::wechat()->success();
    }
}
